/* =================================================================================
 *
 * Fully Scalable Methods for Distributed Tensor Factorization
 * Authors: Kijung Shin (kijungs@cs.cmu.edu), Lee Sael, U Kang
 *
 * Version: 1.0
 * Date: April 10, 2016
 * Main Contact: Kijung Shin (kijungs@cs.cmu.edu)
 *
 * This software is free of charge under research purposes.
 * For commercial purposes, please contact the author.
 *
 * =================================================================================
 */

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Random;

public class Generator {

	/**
	 * create synthetic tensor
	 * 
	 * @param N	dimension
	 * @param I	mode length
	 * @param K	rank
	 * @param omega	number of observable entries
	 * @param outPath
	 * @throws IOException
	 */
	public static void createFactorizableTensor(int N, int I, int K, int omega, String outPath) throws IOException{
		
		Random random = new Random();
		
		float[][][] factors = new float[N][][]; // factor matrices, (n, k, i_{n}) -> A^{(n)}_{i_{n}k}
		for(int dim=0; dim<N; dim++){
			factors[dim] = new float[K][I];
			for(int i=0; i<I; i++){
				for(int r =0; r<K; r++){
					factors[dim][r][i] = (float)random.nextFloat();
				}
			}
		}
		
		BufferedWriter bw = new BufferedWriter(new FileWriter(outPath));
		
		for(int s=0; s<omega; s++){
			while(true){
				int[] index = new int[N];
				for(int dim=0; dim<N; dim++){
					index[dim] = random.nextInt(I);
				}
				float sum = 0;
				for(int r=0; r<K; r++){
					float product = 1;
					for(int dim=0; dim<N; dim++){
						product *= factors[dim][r][index[dim]];
					}
					sum += product;
				}
				
				sum += Math.pow(random.nextFloat(), N); //add random noise
				
				for(int dim=0; dim<N; dim++){
					bw.write(index[dim]+",");
				}
				bw.write(String.format("%f\n",sum + Math.pow(random.nextFloat(), N)));
				break;
			}
		}
		
		bw.close();
	}
	
	public static void main(String[] args) throws IOException{
		
		System.out.println("========================");
		System.out.println("Start parameter check...");
		System.out.println("========================");
		
		String output = args[0];
		System.out.println("-output: "+output);
		int N = Integer.valueOf(args[1]);
		System.out.println("-N: "+N);
		int I = Integer.valueOf(args[2]);
		System.out.println("-I: "+I);
		int K = Integer.valueOf(args[3]);
		System.out.println("-K: "+K);
		int omega = Integer.valueOf(args[4]);
		System.out.println("-omega: "+omega);
		
		System.out.println("========================");
		System.out.println("Start Writing Output...");
		System.out.println("========================");
		
		createFactorizableTensor(N, I, K, omega, output);
		
	}
	
}
